from pylatex import (Document, Package, Command, NewPage, Tabularx
                     #Section, Subsection, Subsubsection, Itemize,  HorizontalSpace, Description, Marker
                    )

#from pylatex.section import Paragraph, Chapter
from pylatex.utils import (#italic, 
                           NoEscape)
from ..report import Markdown, CurrentContainer, ReportText, IPMarkdown, ObjectCode,display
from ..components.mech import en as mech_comp
from ..components.guides import en as guide_comp
from ..components.ode import pl as ode_comp

from ..components.guides.reporting import en as reporting_comp
from ..components.guides.github import en as github_comp
from ..components.guides.systems import en as systems_comp
from ..components.guides.development import en as development_comp
from ..components.guides.pandas import en as pandas_comp
from typing import Optional, List, Union

#from sympy import *
import datetime

class ReportMethods:

    _reported_object = None
    
    @classmethod
    def base_setup(cls):
        
        preliminary_str=(
"""

Examplary setup is as follows:

## CELL 1
## Imports



    from dynpy.utilities.report import *
    from dynpy.utilities.templates.document import Guide

    doc = Guide('./output/report_name')
    

## CELL 2
## Text reporting
    
    #!!! BE SURE ALL PREVIOUS CELLS ARE RUN !!!#
    #!!!       BECAUSE OF NEEDED IMPORTS    !!!#
    
    sec_text = Section('Section that presents text reporting')
    CurrentContainer(sec_text)
    
    display(ReportText('Exemplary text'*100))
    
    #will not work in some projects, restrict usage 
    display(Markdown('Formatted text'*100)) 
    
    
## CELL 3
## Math 

    #!!! BE SURE ALL PREVIOUS CELLS ARE RUN !!!#
    #!!!       BECAUSE OF NEEDED IMPORTS    !!!#

    from sympy import Eq, Symbol, symbols
    
    sec_formula = Section('Section that presents formulas reporting')
    CurrentContainer(sec_formula)
    
    display(ReportText('Mathematical formulas are reported with the support of sympy and it\\'s symbols.'))
    
    a,b = symbols('a b')
    display(SympyFormula(Eq(a,b)))


## CELL 4
## Picture
    
    #!!! BE SURE ALL PREVIOUS CELLS ARE RUN !!!#
    #!!!       BECAUSE OF NEEDED IMPORTS    !!!#
    
    sec_picture = Section('Section that presents pictures reporting')
    CurrentContainer(sec_picture)

    display(Picture('./dynpy/models/images/taipei101.png',caption = 'Caption of picture'))



## CELL 5
## Document

    #!!! BE SURE ALL PREVIOUS CELLS ARE RUN !!!#
    #!!!       BECAUSE OF NEEDED IMPORTS    !!!#

    # Creating file
    # Be sure *output* folder is in the current directory

    guide_name = './output/report_name' #path for report file 

    doc = Guide(guide_name)
    doc.append(sec_text) # adding certain sections
    doc.append(sec_formula)
    doc.append(sec_picture)
    # Generating file
    doc.generate_pdf(clean_tex=True)
    


"""

)
        
        display(IPMarkdown(preliminary_str))    

        
        preliminary_str=(
"""
#Perform basic setup for document creation.

#This method initializes the document and prepares it for content addition.

#Example:

#To prepare a simple document with text and images:

#Good practice here is to allocate 1 section per 1 cell

## ############### CELL 1 ###########################
## Imports

from dynpy.utilities.report import *
from dynpy.utilities.templates.document import Guide

doc = Guide('./output/report_name')
    

## ############### CELL 2 ###########################
## Text reporting
    
sec_text = Section('Section that presents text reporting')
CurrentContainer(sec_text)

display(ReportText('Exemplary text'*100))

#will not work in some projects, restrict usage 
display(Markdown('Formatted text'*100)) 
    
    
## ############### CELL 3 ###########################
## Math 

from sympy import Eq, Symbol, symbols

sec_formula = Section('Section that presents formulas reporting')
CurrentContainer(sec_formula)

display(ReportText('Mathematical formulas are reported with the support of sympy and it\\'s symbols.'))

a,b = symbols('a b')
display(SympyFormula(Eq(a,b)))

## ############### CELL 4 ###########################
## Picture

sec_picture = Section('Section that presents pictures reporting')
CurrentContainer(sec_picture)

display(Picture('./dynpy/models/images/taipei101.png',caption = 'Caption of picture'))



## ############### CELL 5 ###########################
## Document

# Creating file
# Be sure *output* folder is in the current directory

guide_name = './output/report_name' #path for report file 

doc = Guide(guide_name)
doc.append(sec_text) # adding certain sections
doc.append(sec_formula)
doc.append(sec_picture)
# Generating file
doc.generate_pdf(clean_tex=True)

""")    
        return ObjectCode(preliminary_str) 
    
    
    
    @property
    def _report_components(self):
        
        comp_list=[
        mech_comp.TitlePageComponent,
        # mech_comp.SchemeComponent,
        # mech_comp.ExemplaryPictureComponent,
        # mech_comp.KineticEnergyComponent,
        # mech_comp.PotentialEnergyComponent,
        # mech_comp.LagrangianComponent,
        # mech_comp.GoverningEquationComponent,
        # #mech_comp.FundamentalMatrixComponent,
        # mech_comp.GeneralSolutionComponent,
        # #mech_comp.SteadySolutionComponent,
   
        ]
        
        return comp_list
    
    @property
    def default_reported_object(self):
        
        return None

    @property
    def reported_object(self):

        reported_obj=self._reported_object
        
        if reported_obj is None:
            return self.default_reported_object
        else:
            return reported_obj
        

    @reported_object.setter
    def reported_object(self,value):

        self._reported_object=value


    def append_components(self,reported_object=None):

        #self.reported_object = reported_object
        
        doc=self

        for comp in self._report_components:
            doc.append(comp(self.reported_object))

        return None
    

        
class Guide(Document, ReportMethods):
    """
        A class to generate a structured LaTeX document with pre-configured packages and settings.

        Inherits from `Document` and provides additional methods to simplify LaTeX report creation.

        Attributes:
            _documentclass (str): The LaTeX document class (default is 'article').
            latex_name (str): Name of the document.
            packages (List[Union[Package, Command]]): List of LaTeX packages and commands to include in the document.
            _reported_object (Optional[object]): The object being reported on, if any.

        Exemplary Usage:
            >>> from dynpy.utilities.report import *
            >>> from dynpy.utilities.documents.guides import Guide

            >>> doc = Guide('./output/sample_report', title="Sample Report")

            >>> section = Section('Exemplary section name')
            >>> CurrentContainer(section)

            >>> display(Markdown(''' Exemplary Markdown text in the section '''))
            >>> display(ReportText(' Exemplary text appended into section '))

            >>> doc.append(section)

            >>> doc.generate_pdf(clean_tex=True)



        Example of Customization of the document properties:
            >>> from dynpy.utilities.report import *
            >>> from dynpy.utilities.documents.guides import Guide

            >>> custom_geometry = ['lmargin=20mm', 'rmargin=20mm', 'top=25mm', 'bmargin=25mm']

            >>> doc = Guide(
            >>>     default_filepath='./output/custom_report',
            >>>     title='Custom Report',
            >>>     geometry_options=custom_geometry
            >>> )

            >>> section = Section('Exemplary Custom Section Name')

            >>> doc.append(section)

            >>> doc.generate_pdf(clean_tex=True)
    """

    _documentclass = 'article'
    latex_name = 'document'
    packages = [
                  Package('geometry',options=['lmargin=25mm', 'rmargin=25mm',  'top=30mm', 'bmargin=25mm', 'headheight=50mm']),
                  Package('microtype'),
                  Package('authoraftertitle'),
                  Package('polski',options=['MeX']),
                  #Package('geometry',options=['lmargin=25mm', 'rmargin=25mm',  'top=30mm', 'bmargin=25mm', 'headheight=50mm']),
                  Package('listings'),
                  Package('titlesec'),
                  Package('fancyhdr'),
                  Command('pagestyle', arguments=['fancy']),
                  Command('fancyhf', arguments=['']),
                  Command('fancyhead',  arguments=['DynPy Team'],options=['R']),
                  Command('fancyhead', arguments=['Mechanical vibration, 2023'],options=['L']),
                  Command('fancyfoot', arguments=[NoEscape('\\thepage')],options=['C']),
                  ]

    def __init__(
            self,
            default_filepath: str = 'default_filepath',
            title: str = 'Basic title',
            reported_object: Optional[object] = None,
            *,
            documentclass: Optional[str] = None,
            document_options: Optional[List[str]] = None,
            fontenc: str = 'T1',
            inputenc: str = 'utf8',
            font_size: str = 'normalsize',
            lmodern: bool = False,
            textcomp: bool = True,
            microtype: bool = True,
            page_numbers: bool = True,
            indent: Optional[Union[str, int]] = None,
            geometry_options: Optional[List[str]] = None,
            data: Optional[dict] = None,
    ):
        """
            Initialize the Guide class with optional customization.

            Args:
                default_filepath (str): Path for the generated document.
                title (str): Title of the document.
                reported_object (Optional[object]): Object being reported on, if any.
                documentclass (Optional[str]): LaTeX document class (e.g., 'article').
                document_options (Optional[List[str]]): Options for the document class.
                fontenc (str): Font encoding (default 'T1').
                inputenc (str): Input encoding (default 'utf8').
                font_size (str): Font size (default 'normalsize').
                lmodern (bool): Whether to use Latin Modern fonts.
                textcomp (bool): Whether to use the `textcomp` package.
                microtype (bool): Whether to use the `microtype` package.
                page_numbers (bool): Whether to include page numbers.
                indent (Optional[Union[str, int]]): Indentation settings.
                geometry_options (Optional[List[str]]): Geometry options for page layout.
                data (Optional[dict]): Additional data for customization.
        """

        if documentclass is not None: self._documentclass

        self._reported_object = reported_object
        
        super().__init__(
            default_filepath=default_filepath,
            documentclass=self._documentclass,
            document_options=document_options,
            fontenc=fontenc,
            inputenc=inputenc,
            font_size=font_size,
            lmodern=lmodern,
            textcomp=textcomp,
            microtype=microtype,
            page_numbers=page_numbers,
            indent=indent,
            geometry_options=geometry_options,
            data=data,
        )

#         label=self.label
        self.title='Mechanical vibration'
        #self.packages.append(Command('title', arguments=[NoEscape(self.title)]))
        #self.packages.append(Command('author', arguments=['DynPy Team']))
        self.packages.append(Command('date', arguments=[NoEscape('\\today')]))
        #self.append(Command('maketitle'))
        self.append(NewPage())
        # tu implementować co tam potrzeba
        self.append_components()


class ExampleTemplate(Guide):
    pass



class EngeneeringDrawingGuide(Guide):
    
    latex_name = 'document'
    packages = [
                  Package('geometry',options=['lmargin=25mm', 'rmargin=25mm',  'top=30mm', 'bmargin=25mm', 'headheight=50mm']),
                  Package('microtype'),
                  Package('authoraftertitle'),
                  Package('polski',options=['MeX']),
                  #Package('geometry',options=['lmargin=25mm', 'rmargin=25mm',  'top=30mm', 'bmargin=25mm', 'headheight=50mm']),
                  Package('listings'),
                  Package('titlesec'),
                  Package('fancyhdr'),
                  Command('pagestyle', arguments=['fancy']),
                  Command('fancyhf', arguments=['']),
                  Command('fancyhead',  arguments=['DynPy Team'],options=['R']),
                  Command('fancyhead', arguments=['Engeneering Drawing, 2023'],options=['L']),
                  Command('fancyfoot', arguments=[NoEscape('\\thepage')],options=['C']),
    ]
    
    
class DevelopmentGuide(Guide):
    
    latex_name = 'document'
    packages = [
                  Package('geometry',options=['lmargin=25mm', 'rmargin=25mm',  'top=30mm', 'bmargin=25mm', 'headheight=50mm']),
                  Package('microtype'),
                  Package('authoraftertitle'),
                  Package('polski',options=['MeX']),
                  #Package('geometry',options=['lmargin=25mm', 'rmargin=25mm',  'top=30mm', 'bmargin=25mm', 'headheight=50mm']),
                  Package('listings'),
                  Package('titlesec'),
                  Package('fancyhdr'),
                  Command('pagestyle', arguments=['fancy']),
                  Command('fancyhf', arguments=['']),
                  Command('fancyhead',  arguments=['DynPy Team'],options=['R']),
                  Command('fancyhead', arguments=['DynPy development guide, 2023'],options=['L']),
                  Command('fancyfoot', arguments=[NoEscape('\\thepage')],options=['C']),
        ]

        
        
class IntroToCocalcGuideV2(Guide):

    @property
    def _report_components(self):
        
        comp_list=[

#             github_comp.CocalcLoginComponent,
#              development_comp.JupyterSetUpComponent,
#             github_comp.CocalcFolderComponent,

#             github_comp.CocalcDynSysListComponent,
            reporting_comp.ReportingBasicsComponent,

        ]

        return comp_list
        
        
class IntroToCocalcGuide(Guide):

    @property
    def _report_components(self):
        
        comp_list=[

            #github_comp.CocalcLoginComponent,
            github_comp.CocalcUsageComponent,
            development_comp.JupyterSetUpComponent,
            github_comp.CocalcFolderComponent,

            github_comp.CocalcDynSysListComponent,
            reporting_comp.ReportingBasicsComponent,
            

        ]

        return comp_list

class UsageOfDynamicSystemsGuide(Guide):

    @property
    def _report_components(self):

        comp_list=[
            systems_comp.DynamicSystemsUsageIntroComponent,
            systems_comp.DynamicSystemCallComponent,
            systems_comp.DynamicSystemMethodsUsageComponent,

            pandas_comp.NumericalAnalysisSimulationComponent,
            
            systems_comp.SimulationsComponent, # entire component to rewrite
            reporting_comp.SimulationReportComponent,

        ]

        return comp_list
    
    @property
    def default_reported_object(self):
        
        #from ...models.mechanics.tmac import SDOFWinchSystem
        from ...models.mechanics import ForcedSpringMassSystem as DynamicSystem
        
        return DynamicSystem()

class IntroToPandasGuide(Guide):

    @property
    def _report_components(self):

        comp_list=[
            pandas_comp.IntroToPandasUsageComponent,
            pandas_comp.PandasTableGenerationComponent,
            pandas_comp.PandasMethodsComponent,
            reporting_comp.BasicOperationsComponent,
            systems_comp.DynamicSystemCallComponent,
            systems_comp.SimulationsComponent, #Common with *UsageOfDynamicSystemsGuide* class
            reporting_comp.DifferentSimulationsComponent,


        ]

        return comp_list

    @property
    def default_reported_object(self):

        #from ...models.mechanics.tmac import SDOFWinchSystem
        from ...models.mechanics import ForcedSpringMassSystem as DynamicSystem

        return DynamicSystem()
    
    
    
class BasicsOfODESystemGuide(Guide):

    @property
    def _report_components(self):

        comp_list=[
            systems_comp.ODESystemsUsageIntroComponent,
            reporting_comp.BasicUsageOfODESystemComponent,
            #reporting_comp.ODEReportComponent,
            reporting_comp.ReportCompUseComponent,
            reporting_comp.ProjectileExampleComponent,
            systems_comp.ODESimulationComponent,
            systems_comp.ODENumericalSimulationsComponent

        ]

        return comp_list



class BasicsOfDynSysImplementationGuide(UsageOfDynamicSystemsGuide):

    @property
    def _report_components(self):

        comp_list=[
            systems_comp.BasicsOfDynSysImplementationIntroComponent,
            systems_comp.DynSysImplementationComponent,
            systems_comp.DynamicSystemCallComponent,
            systems_comp.DynamicSystemMethodsUsageComponent,
            systems_comp.SimulationsComponent,
            systems_comp.DynSysCodeComponent,

        ]

        return comp_list


class BasicsOfReportingGuide(UsageOfDynamicSystemsGuide):

    @property
    def _report_components(self):

        comp_list=[
            reporting_comp.BasicsOfReportingIntroComponent,
            reporting_comp.ReportingBasicsComponent,
            systems_comp.DynamicSystemCallComponent,
            systems_comp.SimulationsComponent,
            reporting_comp.SimulationReportComponent,
            reporting_comp.ReportingModuleIntroComponent,
            reporting_comp.LibrariesImportComponent,
            reporting_comp.DocumentComponent,
            reporting_comp.CurrentContainerComponent,
            reporting_comp.ReportTextComponent,
#             guide_comp.
            reporting_comp.PictureComponent,
            reporting_comp.SympyFormulaComponent,
            reporting_comp.DocumentGenerationComponent,
            reporting_comp.PredefinedSectionComponent,
#             guide_comp.
            pandas_comp.TablesCreationComponent,
            reporting_comp.AutomarkerIntroComponent,
            reporting_comp.CodeEmbeddingComponent,

            reporting_comp.UnitRegistryIntroComponent,
            reporting_comp.ReportFormattingGuidelinesComponent,
#             guide_comp.

        ]

        return comp_list

   
class ResearchProjectGuidelines(BasicsOfReportingGuide):

    @property
    def _report_components(self):

        comp_list=[


           development_comp.InterimTemplateComponent,
           development_comp.ModellingInPythonGuidelinesComponent,
            # systems_comp.DynamicSystemCallComponent,
            #systems_comp.SimulationsComponent,
            #reporting_comp.SimulationReportComponent,

        ]

        return comp_list 
    
class ThesisGuidelines(ResearchProjectGuidelines):

    @property
    def _report_components(self):

        comp_list=[


           development_comp.InterimTemplateComponent,
           development_comp.ModellingInPythonGuidelinesComponent,
            # systems_comp.DynamicSystemCallComponent,
            #systems_comp.SimulationsComponent,
            #reporting_comp.SimulationReportComponent,

        ]

        return comp_list


    
    
class InterimProjectGuidelines(ResearchProjectGuidelines):

    @property
    def _report_components(self):

        comp_list=[

            development_comp.InterimScheduleComponent,
            development_comp.InterimTemplateComponent,
#             systems_comp.DynamicSystemCallComponent,
#             systems_comp.SimulationsComponent,
#             reporting_comp.SimulationReportComponent,

        ]

        return comp_list
    
    @property
    def default_reported_object(self):

        #from ...models.mechanics.tmac import SDOFWinchSystem
        #from ...models.odes.linear import LinearFirstOrder
        
        return datetime.datetime(2024,7,13)
    
    

    
class IntroDynPyProjectGuidelines(UsageOfDynamicSystemsGuide):

    @property
    def _report_components(self):

        comp_list=[

            development_comp.PythonBasicsGuidelinesComponent,
            development_comp.InterimTemplateComponent,
#             guide_comp.DynamicSystemCallComponent,
#             guide_comp.SimulationsComponent,
#             guide_comp.SimulationReportComponent,

        ]

        return comp_list
    
    
class BasicsOfReportComponentImplementationGuide(UsageOfDynamicSystemsGuide):

    @property
    def _report_components(self):

        comp_list=[

            reporting_comp.ReportCompImplementationComponent,
            reporting_comp.ReportingCompsUsageComponent,
#             reporting_comp.ReportCompImplementationIssueComponent, #Obecnie jest problem z argumentem reported_object, dokładniej classname i sypie błędem
            reporting_comp.ReportingComponentsList

        ]

        return comp_list
    
    @property
    def default_reported_object(self):

        return None
    
class GithubSynchroGuide(UsageOfDynamicSystemsGuide):

    @property
    def _report_components(self):

        comp_list=[
            github_comp.GithubSynchroIntroComponent,
            github_comp.GitSynchroPanelAccessComponent,
            github_comp.GitSynchroIntroComponent,
            github_comp.UsageOfGitHubInterfacesComponent,
            github_comp.UsageOfMeetingCreatorComponent,
            #github_comp.GithubIssueReportComponent, #komponent do listowania issue, chyba nie jest tutaj potrzebny do pokazania

        ]

        return comp_list
    
    @property
    def default_reported_object(self):

        default_data = {'classname':'GitSynchroPanelAccessComponent',
                       'module':'guide.en.py',
                       'field':'guide or report',
                       'target':'`ODESystem` class',
                       'issue_no':123,
                       }


        return default_data



import re
from collections import Counter

class LatexAnalyzerV3:
    def __init__(self, latex_file, output_file=None):
        self.latex_file = latex_file
        self.output_file = output_file
        self.latex_content = None
        self.load_file()
    
    def load_file(self):
        try:
            with open(self.latex_file, 'r', encoding='utf-8') as file:
                self.latex_content = file.read()
        except FileNotFoundError:
            print(f"Error: {self.latex_file} not found.")

    def extract_text_from_commands(self, text):
        patterns = [
            r'\\section\*?\{([^}]*)\}',
            r'\\subsection\*?\{([^}]*)\}',
            r'\\subsubsection\*?\{([^}]*)\}',
            r'\\emph\{([^}]*)\}',
            r'\\caption\{([^}]*)\}'
        ]
        extracted_text = ''
        for pattern in patterns:
            matches = re.findall(pattern, text, re.IGNORECASE)
            extracted_text += ' '.join(matches) + ' '
        return extracted_text
    
    def strip_special_characters_and_commands(self, text):
        text_from_commands = self.extract_text_from_commands(text)
        cleaned_text = re.sub(r'\\[a-zA-Z]+\[[^\]]*\]\{[^}]*\}', '', text)  # Remove commands like \command[...] {...}
        cleaned_text = re.sub(r'\\[a-zA-Z]+\{[^}]*\}', '', cleaned_text)  # Remove commands like \command{...}
        cleaned_text = re.sub(r'\\[a-zA-Z]+', '', cleaned_text)  # Remove commands like \command
        cleaned_text = re.sub(r'[%$\\{}&_^#~]', '', cleaned_text)  # Remove special characters
        return text_from_commands + cleaned_text

    def count_characters(self):
        if self.latex_content:
            cleaned_text = self.strip_special_characters_and_commands(self.latex_content.lower())
            return len(cleaned_text)
        return 0

    def count_words(self, text):
        words = re.findall(r'\b\w+\b', text)
        return len(words)

    def top_words(self, n=10):
        if self.latex_content:
            cleaned_text = self.strip_special_characters_and_commands(self.latex_content.lower())
            words = re.findall(r'\b\w+\b', cleaned_text)
            word_counts = Counter(words)
            return word_counts.most_common(n)
        return []

    def count_words_and_characters_in_sections(self):
        if not self.latex_content:
            return {}

        content = self.latex_content
        sections = re.split(r'(\\section\*?\{[^}]*\})', content)
        counts = {}

        for i in range(1, len(sections), 2):
            section_title = sections[i]
            section_content = sections[i + 1]
            subsections = re.split(r'(\\subsection\*?\{[^}]*\})', section_content)

            section_words = 0
            section_characters = 0

            for j in range(1, len(subsections), 2):
                subsection_title = subsections[j]
                subsection_content = subsections[j + 1]
                subsubsections = re.split(r'(\\subsubsection\*?\{[^}]*\})', subsection_content)

                subsection_words = 0
                subsection_characters = 0

                for k in range(1, len(subsubsections), 2):
                    subsubsection_title = subsubsections[k]
                    subsubsection_content = subsubsections[k + 1]

                    cleaned_subsubsection_content = self.strip_special_characters_and_commands(subsubsection_content)
                    subsubsection_word_count = self.count_words(cleaned_subsubsection_content)
                    subsubsection_character_count = len(cleaned_subsubsection_content)

                    counts[subsubsection_title] = {
                        'words': subsubsection_word_count,
                        'characters': subsubsection_character_count
                    }

                    subsection_words += subsubsection_word_count
                    subsection_characters += subsubsection_character_count

                cleaned_subsection_content = self.strip_special_characters_and_commands(subsection_content)
                subsection_word_count = self.count_words(cleaned_subsection_content)
                subsection_character_count = len(cleaned_subsection_content)

                counts[subsection_title] = {
                    'words': subsection_word_count,
                    'characters': subsection_character_count
                }

                section_words += subsection_word_count
                section_characters += subsection_character_count

            cleaned_section_content = self.strip_special_characters_and_commands(section_content)
            section_word_count = self.count_words(cleaned_section_content)
            section_character_count = len(cleaned_section_content)

            counts[section_title] = {
                'words': section_word_count,
                'characters': section_character_count
            }

        return counts

class DynSysChecker:
    def __init__(self, system, debug= False, verbosity = False):
        self.system = system
        self.runnable = False
        self.debug = debug
        self.verbosity = verbosity
        self.major = [None,None]
        
    def check_init(self):

        if not hasattr(self.system, '__init__'):
            return False

        try:
            _ = self.system()
        except TypeError as e:
            print(e)
            return False
        self.runnable = True
        return True
    
    def check_components(self):

        if not hasattr(self.system, 'components'):
            return False
        
        if self.runnable:
            try:
                comp = self.system().components
            except Exception as e:
                if self.debug:
                    print(e)
                return False
        else:
            try:
                comp = self.system.components
            except Exception as e:
                print(e)
                return False
        if self.verbosity is True:
            return comp
        elif self.verbosity == 'basic':
            comp = str(self.system.components.values()).replace('dict_values([', '').replace('])', '')
            return comp
        else:
            return True
    
    def check_default_data(self):

        if not hasattr(self.system, 'get_default_data'):
            return False
        
        if self.runnable:
            try:
                def_data = self.system().get_default_data()
            except Exception as e:
                if self.debug:
                    print(e)
                return False
        else:
            try:
                def_data = self.system.get_default_data()
            except Exception as e:
                return False
        if self.verbosity is True:
            return def_data
        elif self.verbosity == 'basic':
            if self.major[0] != 'check_numerical_data':
                data_num = len(self.system().get_numerical_data().keys())
            else:
                def_data = self.major[1]
            def_data = len(def_data.keys())
            if data_num > def_data:
                res = f'{str(def_data)} of {str(data_num)}'
            else:
                res = f'{str(data_num)} of {str(def_data)}'
            self.major[0] = 'check_default_data'
            self.major[1] = def_data
            return res
        else:
            return True
        
    def check_numerical_data(self):

        if not hasattr(self.system, 'get_numerical_data'):
            return False
        
        if self.runnable:
            try:
                data_num = self.system().get_numerical_data()
            except Exception as e:
                if self.debug:
                    print(e)
                return False
        else:
            try:
                data_num = self.system.get_numerical_data()
            except Exception:
                return False
        if self.verbosity is True:
            return data_num
        elif self.verbosity == 'basic':
            if self.major[0] != 'check_default_data':
                def_data = len(self.system().get_default_data().keys())
            else:
                def_data = self.major[1]
            data_num = len(data_num.keys())
            if data_num > def_data:
                res = f'{str(def_data)} of {str(data_num)}'
            else:
                res = f'{str(data_num)} of {str(def_data)}'
            self.major[0] = 'check_numerical_data'
            self.major[1] = data_num
            return res
        else:
            return True
        
    def check_symbols_description(self):

        if not hasattr(self.system, 'symbols_description'):
            return False
        
        if self.runnable:
            try:
                sym_desc = self.system().symbols_description()
            except Exception as e:
                if self.debug:
                    print(e)
                return False
        else:
            try:
                sym_desc = self.system.symbols_description()
            except Exception:
                return False
        if self.verbosity is True:
            return sym_desc
        elif self.verbosity == 'basic':
            if self.major[0] != 'check_unit_dict':
                unit_dict = len(self.system().unit_dict().keys())
            else:
                unit_dict = self.major[1]
            sym_desc = len(sym_desc.keys())
            if sym_desc > unit_dict:
                res = f'{str(unit_dict)} of {str(sym_desc)}'
            else:
                res = f'{str(sym_desc)} of {str(unit_dict)}'
            self.major[0] = 'check_symbols_description'
            self.major[1] = sym_desc
            return res
        else:
            return True
    
    def check_unit_dict(self):

        if not hasattr(self.system, 'unit_dict'):
            return False
        
        if self.runnable:
            try:
                unit_dict = self.system().unit_dict()
            except Exception as e:
                if self.debug:
                    print(e)
                return False
        else:
            try:
                unit_dict = self.system.unit_dict()
            except Exception:
                return False
        if self.verbosity is True:
            return unit_dict
        elif self.verbosity == 'basic':
            if self.major[0] != 'check_symbols_description':
                sym_desc = len(self.system().symbols_description().keys())
            else:
                sym_desc = self.major[1]
            unit_dict = len(unit_dict.keys())
            if sym_desc > unit_dict:
                res = f'{str(unit_dict)} of {str(sym_desc)}'
            else:
                res = f'{str(sym_desc)} of {str(unit_dict)}'
            self.major[0] = 'check_unit_dict'
            self.major[1] = unit_dict
            return res
        else:
            return True
    
    def check_equation(self):
        
        if self.runnable:
            try:
                equ = self.system()._eoms[0]
            except Exception as e:
                if self.debug:
                    print(e)
                return False
        else:
            try:
                equ = self.system._eoms[0]
            except Exception:
                return False
        if self.verbosity == True:
            return equ
        elif self.verbosity == 'basic':
            return True
        else:
            return True
        
    def check_picture(self):
        
        if self.runnable:
            try:
                pic = self.system()._as_picture();
                #pic = self.system().preview(example = 'detail_real_name');
            except Exception as e:
                if self.debug:
                    print(e)
                return False
        else:
            try:
                pic = self.system._as_picture();
                #pic = self.system.preview(example = 'detail_real_name');
            except Exception:
                return False
        if self.verbosity:
            return True
        elif self.verbosity == 'basic':
            return True
        else:
            return True
class DynsysCheckerTable:
    def __init__(self, system, verbosity = False, format_for_latex = False):
        import pandas as pd
        initial_data = {'System Name': [], 
                        'Init': [],
                        'Components': [],
                        'Default Data': [],
                        'Numerical Parameters': [],
                        'Symbols Description': [],
                        'Units': [],
                        'Equation': [],   
                        'Picture': [],
                        'Result':[]}
        
        self.df = pd.DataFrame(initial_data)
        pd.set_option("display.max_colwidth", 0)
        #pd.set_option('future.no_silent_downcasting', True)
        
        self.verbosity = verbosity
        self.format_for_latex = format_for_latex
        
        if isinstance(system, __builtins__.__class__):
            self.list_from_Modulestructure(system)
        elif isinstance(system, str):
            self.list_from_Modulestructure(system)
        elif isinstance(system, list):
            self.table_loop(system)
        else:
            self.system = system
            self.table_append(system)
        
        
    def list_from_Modulestructure(self, path):
        from dynpy.utilities.creators import ModuleStructure
        lst = ModuleStructure(path).get_classes()

        import importlib  
        for element in lst:
            m = importlib.import_module(element[0])
            tmp = getattr(m, element[1])
            self.table_append(tmp()) 
    
    def table_append(self, sys):
            
            import pandas as pd
            
            try:
                self.df
            except:
                self.init_table()
            
            param = DynSysChecker(sys, verbosity=self.verbosity)
            
            if self.format_for_latex:
                new_row = {'System Name': sys.__class__.__name__, 
                            'Init': param.check_init(),
                            'Components': str(param.check_components()).replace('_', '\\textunderscore'),
                            'Default Data': str(param.check_default_data()).replace('_', '\textunderscore'),
                            'Numerical Parameters': str(param.check_numerical_data()).replace('_', '\\textunderscore'),
                            'Symbols Description': str(param.check_symbols_description()).replace('_', '\\textunderscore'),
                            'Units': str(param.check_unit_dict()).replace('_', '\\textunderscore'),
                            'Equation': str(param.check_equation()).replace('_', '\\textunderscore'),   
                            'Picture': str(param.check_picture()).replace('_', '\\textunderscore'),
                            'Result': []}
            else: 
                new_row = {'System Name': sys.__class__.__name__, 
                            'Init': param.check_init(),
                            'Components': param.check_components(),
                            'Default Data': param.check_default_data(),
                            'Numerical Parameters': param.check_numerical_data(),
                            'Symbols Description': param.check_symbols_description(),
                            'Units': param.check_unit_dict(),
                            'Equation': param.check_equation(),   
                            'Picture': param.check_picture(),
                            'Result': []}
            if False in new_row.values():
                res = 'To Improve'
            else:
                res = "Correct"
            self.df = pd.concat([self.df, pd.DataFrame([new_row])], ignore_index=True)
            self.df.loc[self.df.index[-1], 'Result'] = res
            
   
    def table_loop(self, sys):
        for element in sys:
            self.table_append(element)
    
    def get_table(self):
        return self.df.replace({1.0: True, 0.0: False}).replace({True: 'Correct', False: 'To Improve'}).replace({'True': 'Correct', 'False': 'To Improve'})